@@ -3,12 +3,11 @@ package Tenjin;
 use Tenjin::Context;
 use Tenjin::Template;
 use Tenjin::Preprocessor;
-use Tenjin::Util;
 
 use strict;
 use warnings;
 
-our $VERSION = 0.052;
+our $VERSION = 0.06;
 our $USE_STRICT = 0;
 our $ENCODING = 'utf8';
 our $BYPASS_TAINT   = 1; # unset if you like taint mode
@@ -17,172 +16,6 @@ our $CONTEXT_CLASS  = 'Tenjin::Context';
 our $PREPROCESSOR_CLASS = 'Tenjin::Preprocessor';
 our $TIMESTAMP_INTERVAL = 10;
 
-sub new {
-	my ($class, $options) = @_;
-
-	my $self = {};
-	foreach (qw[prefix postfix layout path cache preprocess templateclass strict encoding]) {
-		$self->{$_} = delete $options->{$_};
-	}
-	$self->{cache} = 1 unless defined $self->{cache};
-	$self->{init_opts_for_template} = $options;
-	$self->{templates} = {};
-	$self->{prefix} = '' unless $self->{prefix};
-	$self->{postfix} = '' unless $self->{postfix};
-
-	if ($self->{encoding}) {
-		$Tenjin::ENCODING = $self->{encoding};
-	}
-	if (defined $self->{strict}) {
-		$Tenjin::USE_STRICT = $self->{strict};
-	}
-
-	$self->{utils} = Tenjin::Util->new();
-
-	return bless $self, $class;
-}
-
-sub to_filename {
-	my ($self, $template_name) = @_;
-
-	if (substr($template_name, 0, 1) eq ':') {
-		return $self->{prefix} . substr($template_name, 1) . $self->{postfix};
-	}
-
-	return $template_name;
-}
-
-sub find_template_file {
-	my ($self, $filename) = @_;
-
-	my $path = $self->{path};
-	if ($path) {
-		my $sep = $^O eq 'MSWin32' ? '\\\\' : '/';
-		foreach my $dirname (@$path) {
-			my $filepath = $dirname . $sep . $filename;
-			return $filepath if -f $filepath;
-		}
-	} else {
-		return $filename if -f $filename;
-	}
-	my $s = $path ? ("['" . join("','", @$path) . "']") : '[]';
-	die "Tenjin::Engine: \"$filename not found (path=$s)\".";
-}
-
-sub register_template {
-	my ($self, $template_name, $template) = @_;
-
-	$template->{timestamp} = time;
-	$self->{templates}->{$template_name} = $template;
-}
-
-sub get_template {
-	my ($self, $template_name, $context) = @_;
-
-	## get cached template
-	my $template = $self->{templates}->{$template_name};
-
-	## check whether template file is updated or not
-	undef $template if ($template && $template->{filename} && $template->{timestamp} + $TIMESTAMP_INTERVAL <= time);
-
-	## load and register template
-	unless ($template) {
-		my $filename = $self->to_filename($template_name);
-		my $filepath = $self->find_template_file($filename);
-		$template = $self->create_template($filepath, $context);  # $context is passed only for preprocessor
-		$self->register_template($template_name, $template);
-	}
-
-	return $template;
-}
-
-sub read_template_file {
-	my ($self, $template, $filename, $context) = @_;
-
-	if ($self->{preprocess}) {
-		if (! defined($context) || ! $context->{_engine}) {
-			$context ||= {};
-			$context->{'_engine'} = $self;
-		}
-		my $pp = $Tenjin::PREPROCESSOR_CLASS->new();
-		$pp->convert($self->_read_file($filename));
-		return $pp->render($context);
-	}
-
-	return $self->{utils}->read_file($filename, 1);
-}
-
-sub store_cachefile {
-	my ($self, $cachename, $template) = @_;
-
-	my $cache = $template->{script};
-	if (defined $template->{args}) {
-		my $args = $template->{args};
-		$cache = "\#\@ARGS " . join(',', @$args) . "\n" . $cache;
-	}
-	$self->{utils}->write_file($cachename, $cache, 1);
-}
-
-sub load_cachefile {
-	my ($self, $cachename, $template) = @_;
-
-	my $cache = $self->{utils}->read_file($cachename, 1);
-	if ($cache =~ s/\A\#\@ARGS (.*)\r?\n//) {
-		my $argstr = $1;
-		$argstr =~ s/\A\s+|\s+\Z//g;
-		my @args = split(',', $argstr);
-		$template->{args} = \@args;
-	}
-	$template->{script} = $cache;
-}
-
-sub cachename {
-	my ($self, $filename) = @_;
-
-	return $filename . '.cache';
-}
-
-sub create_template {
-	my ($self, $filename, $context) = @_;
-
-	my $cachename = $self->cachename($filename);
-
-	my $class = $self->{templateclass} || $Tenjin::TEMPLATE_CLASS;
-	my $template = $class->new(undef, $self->{init_opts_for_template});
-
-	if (! $self->{cache}) {
-		$template->convert($self->read_template_file($template, $filename, $context), $filename);
-	} elsif (! -f $cachename || (stat $cachename)[9] < (stat $filename)[9]) {
-		$template->convert($self->read_template_file($template, $filename, $context), $filename);
-		$self->store_cachefile($cachename, $template);
-	} else {
-		$template->{filename} = $filename;
-		$self->load_cachefile($cachename, $template);
-	}
-	$template->compile();
-
-	return $template;
-}
-
-sub render {
-	my ($self, $template_name, $context) = @_;
-
-	$context ||= {};
-	$context->{'_engine'} = $self;
-
-	my $template = $self->get_template($template_name, $context); # pass $context only for preprocessing
-	my $output = $template->_render($context);
-	die("*** ERROR: $template->{filename}\n", $@) if $@;
-
-	return $output;
-}
-
-__PACKAGE__;
-
-__END__
-
-=pod
-
 =head1 NAME
 
 Tenjin - Fast templating engine with support for embedded Perl.
@@ -208,7 +41,7 @@ Tenjin - Fast templating engine with support for embedded Perl.
 
 =head1 VERSION
 
-0.052
+0.06
 
 =head1 DESCRIPTION
 
@@ -249,7 +82,7 @@ of this module is NOT backwards compatible with previous versions.
 
 =head1 METHODS
 
-=head2 new \%options
+=head2 new( \%options )
 
 This creates a new instant of Tenjin. C<\%options> is a hash-ref
 containing Tenjin's configuration options:
@@ -259,32 +92,57 @@ containing Tenjin's configuration options:
 =item * B<path> - Array-ref of filesystem paths where templates will be searched
 
 =item * B<prefix> - A string that will be automatically prepended to template names
-		 when searching for them in the path. Empty by default.
+when searching for them in the path. Empty by default.
 
 =item * B<postfix> - The default extension to be automtically appended to template names
-		  when searching for them in the path. Don't forget to include the
-		  dot, such as '.html'. Empty by default.
+when searching for them in the path. Don't forget to include the
+dot, such as '.html'. Empty by default.
 
 =item * B<cache> - If set to 1 (the default), compiled templates will be cached on the
-		filesystem.
+filesystem (this means the template's code will be cached, not the completed rendered
+output).
 
 =item * B<preprocess> - Enable template preprocessing (turned off by default). Only
-		     use if you're actually using any preprocessed Perl code in
-		     your templates.
+use if you're actually using any preprocessed Perl code in your templates.
 
 =item * B<layout> - Name of a layout template that can be optionally used. If set,
-		 templates will be automatically inserted into the layout template,
-		 in the location where you use C<[== $_content ==]>.
+templates will be automatically inserted into the layout template,
+in the location where you use C<[== $_content ==]>.
 
 =item * B<strict> - Another way to make Tenjin use strict on embedded Perl code (turned
-		 off by default).
+off by default).
 
 =item * B<encoding> - Another way to set the encoding of your template files (set to utf8
-		   by default).
+by default).
 
 =back
 
-=head2 render( $tmpl_name, [\%context, $layout] )
+=cut
+
+sub new {
+	my ($class, $options) = @_;
+
+	my $self = {};
+	foreach (qw[prefix postfix layout path cache preprocess templateclass strict encoding]) {
+		$self->{$_} = delete $options->{$_};
+	}
+	$self->{cache} = 1 unless defined $self->{cache};
+	$self->{init_opts_for_template} = $options;
+	$self->{templates} = {};
+	$self->{prefix} = '' unless $self->{prefix};
+	$self->{postfix} = '' unless $self->{postfix};
+
+	if ($self->{encoding}) {
+		$Tenjin::ENCODING = $self->{encoding};
+	}
+	if (defined $self->{strict}) {
+		$Tenjin::USE_STRICT = $self->{strict};
+	}
+
+	return bless $self, $class;
+}
+
+=head2 render( $tmpl_name, [\%context, $use_layout] )
 
 Renders a template whose name is identified by C<$tmpl_name>. Remember that a prefix
 and a postfix might be added if they where set when creating the Tenjin instance.
@@ -293,14 +151,257 @@ C<$context> is a hash-ref containing the variables that will be available for us
 the templates. So, for example, if your C<\%context> is { message => 'Hi there }, then
 you can use C<$message> inside your templates.
 
-C<$layout> is a flag denoting whether or not to render this template into the layout template
-there was set when creating the Tenjin instance.
+C<$use_layout> is a flag denoting whether or not to render this template into a layout
+template (when doing so, the template will be rendered, then the rendered output will be
+added to the context hash-ref as '_content', and finally the layout template will be rendered
+with the revised context and returned. If C<$use_layout> is 1, than Tenjin will use the
+layout template that was set when creating the Tenjin instance (via the 'layout' configuration
+option). If you want to use a different layout template (or if you haven't defined a layout
+template when creating the Tenjin instance), then you must add the layout template's name
+to the context as '_layout'. You can also just pass the layout template's name as C<$use_layout>,
+which has precendence over C<< $context->{_layout} >>. If C<$use_layout> is 0 or undefined,
+then a layout template will not be used, even if C<< $context->{_layout} >> is defined.
 
-Please note that file templates are cached on disk (with a '.cache') extension.
+Please note that by default file templates are cached on disk (with a '.cache') extension.
 Tenjin automatically deprecates these cache files every 10 seconds. If you
 find this value is too low, you can override the C<$Tenjin::TIMESTAMP_INTERVAL>
 variable with your preferred value.
 
+=cut
+
+sub render {
+	my ($self, $template_name, $context, $use_layout) = @_;
+
+	$context ||= {};
+	$context->{'_engine'} = $self;
+
+	my $template = $self->get_template($template_name, $context); # pass $context only for preprocessing
+	my $output = $template->_render($context);
+	die("*** ERROR: $template->{filename}\n", $@) if $@;
+
+	# should we render inside a layout template?
+	if ($use_layout) {
+		# was a layout template name passed, or should we use the layout defined
+		# in when creating the engine instance?
+		my $layout_tmpl = $use_layout =~ m/^1$/ ? $self->{layout} : $use_layout;
+		$layout_tmpl ||= $context->{_layout};
+		
+		# make sure we have a layout template to render
+		return $output unless $layout_tmpl;
+
+		# add the output of the rendered template to the context as '_content'
+		# and remove the reference to the layout from the context (if present)
+		$context->{_content} = $output;
+		delete $context->{_layout};
+		
+		# render the layout template
+		$output = $self->get_template($layout_tmpl, $context)->_render($context);
+		die("*** ERROR: $layout_tmpl\n", $@) if $@;
+	}
+
+	return $output;
+}
+
+=head2 register_template( $template_name, $template )
+
+Receives the name of a template and its L<Tenjin::Template> object
+and stores it in memory for usage by the engine.
+
+=cut
+
+sub register_template {
+	my ($self, $template_name, $template) = @_;
+
+	$template->{timestamp} = time;
+	$self->{templates}->{$template_name} = $template;
+}
+
+=head1 INTERNAL METHODS
+
+=head2 get_template( $template_name, $context )
+
+Receives the name of a template and the context object and tries to find
+that template in the engine's memory. If it's not there, it will try to find
+it in the file system (the cache file might be loaded, if present). Returns
+the templates L<Tenjin::Template> object.
+
+=cut
+
+sub get_template {
+	my ($self, $template_name, $context) = @_;
+
+	## get cached template
+	my $template = $self->{templates}->{$template_name};
+
+	## check whether template file is updated or not
+	undef $template if ($template && $template->{filename} && $template->{timestamp} + $TIMESTAMP_INTERVAL <= time);
+
+	## load and register template
+	unless ($template) {
+		my $filename = $self->to_filename($template_name);
+		my $filepath = $self->find_template_file($filename);
+		$template = $self->create_template($filepath, $context);  # $context is passed only for preprocessor
+		$self->register_template($template_name, $template);
+	}
+
+	return $template;
+}
+
+=head2 to_filename( $template_name )
+
+Receives a template name and returns the proper file name to be searched
+in the file system, which will only be different than C<$template_name>
+if it begins with ':', in which case the prefix and postfix configuration
+options will be appended and prepended to the template name (minus the ':'),
+respectively.
+
+=cut
+
+sub to_filename {
+	my ($self, $template_name) = @_;
+
+	if (substr($template_name, 0, 1) eq ':') {
+		return $self->{prefix} . substr($template_name, 1) . $self->{postfix};
+	}
+
+	return $template_name;
+}
+
+=head2 find_template_file( $filename )
+
+Receives a template filename and searches for it in the path defined in
+the configuration options (or, if a path was not set, in the current
+working directory). Returns the absolute path to the file.
+
+=cut
+
+sub find_template_file {
+	my ($self, $filename) = @_;
+
+	my $path = $self->{path};
+	if ($path) {
+		my $sep = $^O eq 'MSWin32' ? '\\\\' : '/';
+		foreach my $dirname (@$path) {
+			my $filepath = $dirname . $sep . $filename;
+			return $filepath if -f $filepath;
+		}
+	} else {
+		return $filename if -f $filename;
+	}
+	my $s = $path ? ("['" . join("','", @$path) . "']") : '[]';
+	die "Tenjin::Engine: \"$filename not found (path=$s)\".";
+}
+
+=head2 read_template_file( $template, $filename, $context )
+
+Receives a template object and its absolute file path and reads that file.
+If preprocessing is on, preprocessing will take place using the provided
+context object.
+
+=cut
+
+sub read_template_file {
+	my ($self, $template, $filename, $context) = @_;
+
+	if ($self->{preprocess}) {
+		if (! defined($context) || ! $context->{_engine}) {
+			$context ||= {};
+			$context->{'_engine'} = $self;
+		}
+		my $pp = $Tenjin::PREPROCESSOR_CLASS->new();
+		$pp->convert($template->_read_file($filename));
+		return $pp->render($context);
+	}
+
+	return $template->_read_file($filename, 1);
+}
+
+=head2 cachename( $filename )
+
+Receives a template filename and returns its standard cache filename (which
+will simply be C<$filename> with '.cache' appended to it.
+
+=cut
+
+sub cachename {
+	my ($self, $filename) = @_;
+
+	return $filename . '.cache';
+}
+
+=head2 store_cachefile( $cachename, $template )
+
+Receives the name of a template cache file and the corrasponding template
+object, and creates the cache file on disk.
+
+=cut
+
+sub store_cachefile {
+	my ($self, $cachename, $template) = @_;
+
+	my $cache = $template->{script};
+	if (defined $template->{args}) {
+		my $args = $template->{args};
+		$cache = "\#\@ARGS " . join(',', @$args) . "\n" . $cache;
+	}
+	$template->_write_file($cachename, $cache, 1);
+}
+
+=head2 load_cachefile( $cachename, $template )
+
+Receives the name of a template cache file and the corrasponding template
+object, reads the cache file and stores it in the template object (as 'script').
+
+=cut
+
+sub load_cachefile {
+	my ($self, $cachename, $template) = @_;
+
+	my $cache = $template->_read_file($cachename, 1);
+	if ($cache =~ s/\A\#\@ARGS (.*)\r?\n//) {
+		my $argstr = $1;
+		$argstr =~ s/\A\s+|\s+\Z//g;
+		my @args = split(',', $argstr);
+		$template->{args} = \@args;
+	}
+	$template->{script} = $cache;
+}
+
+=head2 create_template( $filename, $context )
+
+Receives an absolute path to a template file and the context object, reads
+the file, processes it (which may involve loading the template's cache file
+or creating the template's cache file), compiles it and returns the template
+object.
+
+=cut
+
+sub create_template {
+	my ($self, $filename, $context) = @_;
+
+	my $cachename = $self->cachename($filename);
+
+	my $class = $self->{templateclass} || $Tenjin::TEMPLATE_CLASS;
+	my $template = $class->new(undef, $self->{init_opts_for_template});
+
+	if (! $self->{cache}) {
+		$template->convert($self->read_template_file($template, $filename, $context), $filename);
+	} elsif (! -f $cachename || (stat $cachename)[9] < (stat $filename)[9]) {
+		$template->convert($self->read_template_file($template, $filename, $context), $filename);
+		$self->store_cachefile($cachename, $template);
+	} else {
+		$template->{filename} = $filename;
+		$self->load_cachefile($cachename, $template);
+	}
+	$template->compile();
+
+	return $template;
+}
+
+__PACKAGE__;
+
+__END__
+
 =head1 SEE ALSO
 
 The original Tenjin website is located at L<http://www.kuwata-lab.com/tenjin/>. In there check out
@@ -312,17 +413,17 @@ Note that the Perl version of Tenjin is refered to as plTenjin on the Tenjin web
 and that, as oppose to this module, the website suggests using a .plhtml extension
 for the templates instead of .html (this is entirely your choice).
 
-L<Tenjin::Template>, L<Catalyst::View::Tenjin>.
+L<Tenjin::Template>, L<Catalyst::View::Tenjin>, L<Dancer::Template::Tenjin>.
 
 =head1 CHANGES
 
-This version of Tenjin breaks backwards compatibility with previous versions.
+Version 0.05 of this module broke backwards compatibility with previous versions.
 In particular, the Tenjin::Engine module does not exist any more and is
 instead integrated into this one. Templates are also rendered entirely
 different (as per changes in the original tenjin) which provides much
 faster rendering.
 
-Upon upgrading to this version, you MUST perform the following changes
+Upon upgrading to versions 0.05 and above, you MUST perform the following changes
 for your applications (or, if you're using Catalyst, you must also upgrade
 L<Catalyst::View::Tenjin>):
 
@@ -338,6 +439,11 @@ templates structure and WILL cause your application to fail if present.
 
 =back
 
+Version 0.06 (this version) restored the layout template feature which was
+accidentaly missing in version 0.05, and the ability to call the utility
+methods of L<Tenjin::Util> natively inside templates. You will want to
+remove your templates' .cache files when upgrading to 0.6 too.
+
 =head1 TODO
 
 =over
