@@ -4,8 +4,9 @@ use strict;
 use warnings;
 use Tenjin;
 
-my $t = Tenjin->new({ path => ['./'], postfix => '.html' });
-print $t->render('test_tmpl.html', {
+my $t = Tenjin->new({ path => ['./'], postfix => '.html', layout => 'test_layout_tmpl.html' });
+
+my $context = {
 	scalar_variable		=> 'hello',
 	hash_variable		=> { hash_value_key => 'sensible' },
 	array_variable		=> [ undef, undef, 'world' ],
@@ -20,4 +21,11 @@ print $t->render('test_tmpl.html', {
 	variable_expression_a	=> 2,
 	variable_expression_b	=> 5,
 	variable_function_arg	=> 'asf asdf asdfff',
-});
+};
+
+print	"Standalone rendering of test_tmpl.html:\n",		$t->render('test_tmpl.html', $context);
+print	"\n\n\nRendered inside test_layout_tmpl.html:\n",	$t->render('test_tmpl.html', $context, 1);
+print	"\n\n\nRendered inside test_layout_tmpl_2.html:\n",	$t->render('test_tmpl.html', $context, 'test_layout_tmpl_2.html');
+print	"\n\n\nUsing Tenjin Utility methods:\n",		$t->render('test_util.html', $context), "\n";
+print	"\n\n\nCapture and Placeholder:\n",			$t->render('test_capt.html', $context), "\n";
+
